package family.flatfsmsimultaneous.adapters.flatfsmmt;

import family.flatfsmsimultaneous.adapters.flatfsmmt.FinalStateAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.ForkAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.InitialStateAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.JoinAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.NamedElementAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.PseudostateAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.StateAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.StateMachineAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.TimedTransitionAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.TransitionAdapter;
import family.flatfsmsimultaneous.adapters.flatfsmmt.TriggerAdapter;
import flatfsm.fsm.FinalState;
import flatfsm.fsm.Fork;
import flatfsm.fsm.InitialState;
import flatfsm.fsm.Join;
import flatfsm.fsm.NamedElement;
import flatfsm.fsm.Pseudostate;
import flatfsm.fsm.State;
import flatfsm.fsm.StateMachine;
import flatfsm.fsm.TimedTransition;
import flatfsm.fsm.Transition;
import flatfsm.fsm.Trigger;
import fr.inria.diverse.melange.adapters.AdaptersFactory;
import fr.inria.diverse.melange.adapters.EObjectAdapter;
import java.util.WeakHashMap;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

@SuppressWarnings("all")
public class FlatFsmMTAdaptersFactory implements AdaptersFactory {
  private static FlatFsmMTAdaptersFactory instance;
  
  private WeakHashMap<EObject, EObjectAdapter> register;
  
  public FlatFsmMTAdaptersFactory() {
    register = new WeakHashMap();
  }
  
  public static FlatFsmMTAdaptersFactory getInstance() {
    if (instance == null) {
    	instance = new family.flatfsmsimultaneous.adapters.flatfsmmt.FlatFsmMTAdaptersFactory() ;
    }
    return instance ;
  }
  
  public EObjectAdapter createAdapter(final EObject o, final Resource res) {
    if (o instanceof flatfsm.fsm.StateMachine){
    	return createStateMachineAdapter((flatfsm.fsm.StateMachine) o, res) ;
    }
    if (o instanceof flatfsm.fsm.FinalState){
    	return createFinalStateAdapter((flatfsm.fsm.FinalState) o, res) ;
    }
    if (o instanceof flatfsm.fsm.InitialState){
    	return createInitialStateAdapter((flatfsm.fsm.InitialState) o, res) ;
    }
    if (o instanceof flatfsm.fsm.State){
    	return createStateAdapter((flatfsm.fsm.State) o, res) ;
    }
    if (o instanceof flatfsm.fsm.TimedTransition){
    	return createTimedTransitionAdapter((flatfsm.fsm.TimedTransition) o, res) ;
    }
    if (o instanceof flatfsm.fsm.Transition){
    	return createTransitionAdapter((flatfsm.fsm.Transition) o, res) ;
    }
    if (o instanceof flatfsm.fsm.Fork){
    	return createForkAdapter((flatfsm.fsm.Fork) o, res) ;
    }
    if (o instanceof flatfsm.fsm.Join){
    	return createJoinAdapter((flatfsm.fsm.Join) o, res) ;
    }
    if (o instanceof flatfsm.fsm.Pseudostate){
    	return createPseudostateAdapter((flatfsm.fsm.Pseudostate) o, res) ;
    }
    if (o instanceof flatfsm.fsm.NamedElement){
    	return createNamedElementAdapter((flatfsm.fsm.NamedElement) o, res) ;
    }
    if (o instanceof flatfsm.fsm.Trigger){
    	return createTriggerAdapter((flatfsm.fsm.Trigger) o, res) ;
    }
    
    return null ;
  }
  
  public NamedElementAdapter createNamedElementAdapter(final NamedElement adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.NamedElementAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.NamedElementAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.NamedElementAdapter) adapter ;
    }
  }
  
  public StateMachineAdapter createStateMachineAdapter(final StateMachine adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.StateMachineAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.StateMachineAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.StateMachineAdapter) adapter ;
    }
  }
  
  public StateAdapter createStateAdapter(final State adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.StateAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.StateAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.StateAdapter) adapter ;
    }
  }
  
  public FinalStateAdapter createFinalStateAdapter(final FinalState adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.FinalStateAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.FinalStateAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.FinalStateAdapter) adapter ;
    }
  }
  
  public InitialStateAdapter createInitialStateAdapter(final InitialState adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.InitialStateAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.InitialStateAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.InitialStateAdapter) adapter ;
    }
  }
  
  public TransitionAdapter createTransitionAdapter(final Transition adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.TransitionAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.TransitionAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.TransitionAdapter) adapter ;
    }
  }
  
  public TimedTransitionAdapter createTimedTransitionAdapter(final TimedTransition adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.TimedTransitionAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.TimedTransitionAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.TimedTransitionAdapter) adapter ;
    }
  }
  
  public TriggerAdapter createTriggerAdapter(final Trigger adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.TriggerAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.TriggerAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.TriggerAdapter) adapter ;
    }
  }
  
  public PseudostateAdapter createPseudostateAdapter(final Pseudostate adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.PseudostateAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.PseudostateAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.PseudostateAdapter) adapter ;
    }
  }
  
  public ForkAdapter createForkAdapter(final Fork adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.ForkAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.ForkAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.ForkAdapter) adapter ;
    }
  }
  
  public JoinAdapter createJoinAdapter(final Join adaptee, final Resource res) {
    if (adaptee == null)
    	return null;
    EObjectAdapter adapter = register.get(adaptee);
    if(adapter != null)
    	 return (family.flatfsmsimultaneous.adapters.flatfsmmt.JoinAdapter) adapter;
    else {
    	adapter = new family.flatfsmsimultaneous.adapters.flatfsmmt.JoinAdapter() ;
    	adapter.setAdaptee(adaptee) ;
    	adapter.setResource(res) ;
    	register.put(adaptee, adapter);
    	return (family.flatfsmsimultaneous.adapters.flatfsmmt.JoinAdapter) adapter ;
    }
  }
}
