package semantics.composite.simultaneous;

import com.google.common.collect.Iterables;
import compositefsm.fsm.CompositeState;
import compositefsm.fsm.Fork;
import compositefsm.fsm.InitialState;
import compositefsm.fsm.Join;
import compositefsm.fsm.Region;
import compositefsm.fsm.StateMachine;
import compositefsm.fsm.Transition;
import compositefsm.fsm.Trigger;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import semantics.common.Context;
import semantics.composite.simultaneous.CompositeStateAspect;
import semantics.composite.simultaneous.ForkThread;
import semantics.composite.simultaneous.StateAspect;
import semantics.composite.simultaneous.StateMachineAspect;

/**
 * Class for executing events in parallel.
 */
@SuppressWarnings("all")
public class SimultaneousEventsThread extends Thread {
  private String event;
  
  private StateMachine stateMachine;
  
  private Context context;
  
  private ArrayList<compositefsm.fsm.State> currentState;
  
  private ArrayList<Transition> currentTransitions;
  
  /**
   * Constructor of the class. Initializes the attributes with the values
   * in the parameters
   */
  public SimultaneousEventsThread(final String _event, final StateMachine _stateMachine, final Context _context) {
    this.event = _event;
    this.stateMachine = _stateMachine;
    this.context = _context;
    ArrayList<compositefsm.fsm.State> _arrayList = new ArrayList<compositefsm.fsm.State>();
    this.currentState = _arrayList;
    ArrayList<Transition> _arrayList_1 = new ArrayList<Transition>();
    this.currentTransitions = _arrayList_1;
  }
  
  /**
   * Runs the thread!
   */
  @Override
  public void run() {
    EList<compositefsm.fsm.State> attendedStates = new BasicEList<compositefsm.fsm.State>();
    EList<compositefsm.fsm.State> _currentState = StateMachineAspect.currentState(this.stateMachine);
    for (final compositefsm.fsm.State _state : _currentState) {
      EList<Transition> _outgoing = _state.getOutgoing();
      for (final Transition transition : _outgoing) {
        boolean _and = false;
        Trigger _trigger = transition.getTrigger();
        boolean _tripleNotEquals = (_trigger != null);
        if (!_tripleNotEquals) {
          _and = false;
        } else {
          Trigger _trigger_1 = transition.getTrigger();
          String _expression = _trigger_1.getExpression();
          boolean _equals = _expression.equals(this.event);
          _and = _equals;
        }
        if (_and) {
          this.currentState.add(_state);
        }
      }
    }
    for (final compositefsm.fsm.State _state_1 : this.currentState) {
      EList<Transition> _outgoing_1 = _state_1.getOutgoing();
      for (final Transition transition_1 : _outgoing_1) {
        Trigger _trigger_2 = transition_1.getTrigger();
        String _expression_1 = _trigger_2.getExpression();
        boolean _equals_1 = _expression_1.equals(this.event);
        if (_equals_1) {
          attendedStates.add(_state_1);
          this.currentTransitions.add(transition_1);
          if ((_state_1 instanceof CompositeState)) {
            final CompositeState composite = ((CompositeState)_state_1);
            EList<compositefsm.fsm.State> _allStates = CompositeStateAspect.getAllStates(composite);
            attendedStates.addAll(_allStates);
          }
        }
      }
    }
    this.currentState.removeAll(attendedStates);
    StateMachineAspect.removeCurrentStates(this.stateMachine, attendedStates);
    this.processTransitions(this.context);
  }
  
  /**
   * Processes the current active transitions by activating their target states
   * and executing the steps on the states (should the evaluation of the states be called from here?).
   */
  private void processTransitions(final Context context) {
    for (final Transition _transition : this.currentTransitions) {
      {
        final compositefsm.fsm.State target = _transition.getTarget();
        if ((target instanceof Fork)) {
          ArrayList<ForkThread> threads = new ArrayList<ForkThread>();
          compositefsm.fsm.State _target = _transition.getTarget();
          EList<Transition> _outgoing = _target.getOutgoing();
          for (final Transition _forkTransition : _outgoing) {
            {
              compositefsm.fsm.State _target_1 = _forkTransition.getTarget();
              this.currentState.add(_target_1);
              EList<compositefsm.fsm.State> _currentState = StateMachineAspect.currentState(this.stateMachine);
              compositefsm.fsm.State _target_2 = _forkTransition.getTarget();
              _currentState.add(_target_2);
              compositefsm.fsm.State _target_3 = _forkTransition.getTarget();
              ForkThread _forkThread = new ForkThread(_target_3, context);
              threads.add(_forkThread);
              _forkThread.start();
            }
          }
          boolean threadsAlive = true;
          while (threadsAlive) {
            {
              int stillAlive = 0;
              for (final Thread _thread : threads) {
                boolean _isAlive = _thread.isAlive();
                if (_isAlive) {
                  stillAlive++;
                }
              }
              if ((stillAlive == 0)) {
                threadsAlive = false;
              }
            }
          }
        } else {
          if ((target instanceof CompositeState)) {
            this.currentState.add(target);
            EList<compositefsm.fsm.State> _currentState = StateMachineAspect.currentState(this.stateMachine);
            _currentState.add(target);
            final CompositeState composite = ((CompositeState)target);
            EList<Region> _regions = composite.getRegions();
            final Function1<Region, EList<compositefsm.fsm.State>> _function = (Region r) -> {
              return r.getStates();
            };
            List<EList<compositefsm.fsm.State>> _map = ListExtensions.<Region, EList<compositefsm.fsm.State>>map(_regions, _function);
            Iterable<compositefsm.fsm.State> _flatten = Iterables.<compositefsm.fsm.State>concat(_map);
            final Function1<compositefsm.fsm.State, Boolean> _function_1 = (compositefsm.fsm.State s) -> {
              return Boolean.valueOf((s instanceof InitialState));
            };
            final Iterable<compositefsm.fsm.State> initStates = IterableExtensions.<compositefsm.fsm.State>filter(_flatten, _function_1);
            final ArrayList<ForkThread> threads_1 = new ArrayList<ForkThread>();
            final Consumer<compositefsm.fsm.State> _function_2 = (compositefsm.fsm.State init) -> {
              this.currentState.add(init);
              EList<compositefsm.fsm.State> _currentState_1 = StateMachineAspect.currentState(this.stateMachine);
              _currentState_1.add(init);
              ForkThread _forkThread = new ForkThread(init, context);
              threads_1.add(_forkThread);
              _forkThread.start();
            };
            initStates.forEach(_function_2);
            boolean threadsAlive_1 = true;
            while (threadsAlive_1) {
              {
                int stillAlive = 0;
                for (final Thread _thread : threads_1) {
                  boolean _isAlive = _thread.isAlive();
                  if (_isAlive) {
                    stillAlive++;
                  }
                }
                if ((stillAlive == 0)) {
                  threadsAlive_1 = false;
                }
              }
            }
          } else {
            compositefsm.fsm.State _target_1 = _transition.getTarget();
            this.currentState.add(_target_1);
            EList<compositefsm.fsm.State> _currentState_1 = StateMachineAspect.currentState(this.stateMachine);
            compositefsm.fsm.State _target_2 = _transition.getTarget();
            _currentState_1.add(_target_2);
            StateAspect.eval(target, context);
            CompositeState parent = target.getParentState();
            while ((((parent != null) && (!this.currentState.contains(parent))) && (!StateMachineAspect.currentState(this.stateMachine).contains(parent)))) {
              {
                this.currentState.add(parent);
                EList<compositefsm.fsm.State> _currentState_2 = StateMachineAspect.currentState(this.stateMachine);
                _currentState_2.add(parent);
                CompositeState _parentState = parent.getParentState();
                parent = _parentState;
              }
            }
          }
        }
      }
    }
    EList<compositefsm.fsm.State> toAdd = new BasicEList<compositefsm.fsm.State>();
    EList<compositefsm.fsm.State> toRemove = new BasicEList<compositefsm.fsm.State>();
    for (final compositefsm.fsm.State _state : this.currentState) {
      boolean _and = false;
      EList<Transition> _outgoing = _state.getOutgoing();
      int _size = _outgoing.size();
      boolean _greaterThan = (_size > 0);
      if (!_greaterThan) {
        _and = false;
      } else {
        EList<Transition> _outgoing_1 = _state.getOutgoing();
        Transition _get = _outgoing_1.get(0);
        compositefsm.fsm.State _target = _get.getTarget();
        _and = (_target instanceof Join);
      }
      if (_and) {
        boolean _and_1 = false;
        EList<Transition> _outgoing_2 = _state.getOutgoing();
        Transition _get_1 = _outgoing_2.get(0);
        compositefsm.fsm.State _target_1 = _get_1.getTarget();
        boolean _contains = toAdd.contains(_target_1);
        boolean _not = (!_contains);
        if (!_not) {
          _and_1 = false;
        } else {
          EList<Transition> _outgoing_3 = _state.getOutgoing();
          Transition _get_2 = _outgoing_3.get(0);
          compositefsm.fsm.State _target_2 = _get_2.getTarget();
          boolean _contains_1 = this.currentState.contains(_target_2);
          boolean _not_1 = (!_contains_1);
          _and_1 = _not_1;
        }
        if (_and_1) {
          EList<Transition> _outgoing_4 = _state.getOutgoing();
          Transition _get_3 = _outgoing_4.get(0);
          compositefsm.fsm.State _target_3 = _get_3.getTarget();
          toAdd.add(_target_3);
        }
        boolean _contains_2 = toRemove.contains(_state);
        boolean _not_2 = (!_contains_2);
        if (_not_2) {
          toRemove.add(_state);
        }
      }
    }
    this.currentState.removeAll(toRemove);
    this.currentState.addAll(toAdd);
    StateMachineAspect.removeCurrentStates(this.stateMachine, toRemove);
    StateMachineAspect.addCurrentStates(this.stateMachine, toAdd);
  }
}
