package semantics.flat;

import com.itextpdf.awt.DefaultFontMapper;
import com.itextpdf.text.Document;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfTemplate;
import com.itextpdf.text.pdf.PdfWriter;
import flatfsm.fsm.Fork;
import flatfsm.fsm.Join;
import flatfsm.fsm.State;
import flatfsm.fsm.StateMachine;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.util.Date;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.data.category.IntervalCategoryDataset;
import org.jfree.data.gantt.Task;
import org.jfree.data.gantt.TaskSeries;
import org.jfree.data.gantt.TaskSeriesCollection;
import org.jfree.data.time.SimpleTimePeriod;

@SuppressWarnings("all")
public class Printer {
  /**
   * Prints time final state of the states machine i.e., the schedule after the evaluation
   */
  public void printFinalStateInConsole(final StateMachine stateMachine) {
    EList<State> _states = stateMachine.getStates();
    for (final State _state : _states) {
      if (((!(_state instanceof Fork)) && (!(_state instanceof Join)))) {
        String _name = _state.getName();
        String _plus = ("   - State: " + _name);
        String _plus_1 = (_plus + " [initialTime: ");
        int _initialTime = _state.getInitialTime();
        String _plus_2 = (_plus_1 + Integer.valueOf(_initialTime));
        String _plus_3 = (_plus_2 + " finalTime: ");
        int _finalTime = _state.getFinalTime();
        String _plus_4 = (_plus_3 + Integer.valueOf(_finalTime));
        String _plus_5 = (_plus_4 + "]");
        InputOutput.<String>println(_plus_5);
      }
    }
  }
  
  /**
   * Prints as a Gantt diagram the final state of the states machine
   * i.e., the schedule after the evaluation
   */
  public void printFinalStateInFile(final StateMachine stateMachine, final String filePath) {
    IntervalCategoryDataset dataset = Printer.createDataset(stateMachine);
    JFreeChart chart = this.createChart(dataset);
    try {
      this.saveChartToPDF(chart, filePath, 500, 300);
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception exc = (Exception)_t;
        throw new RuntimeException("auto-generated try/catch", exc);
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
  }
  
  /**
   * Returns the Gantt model according to the final state of the state machine.
   */
  public static IntervalCategoryDataset createDataset(final StateMachine stateMachine) {
    TaskSeries s1 = new TaskSeries("State Machine");
    EList<State> _states = stateMachine.getStates();
    for (final State _state : _states) {
      if (((!(_state instanceof Fork)) && (!(_state instanceof Join)))) {
        String _name = _state.getName();
        int _initialTime = _state.getInitialTime();
        Date _date = new Date(_initialTime);
        int _finalTime = _state.getFinalTime();
        Date _date_1 = new Date(_finalTime);
        SimpleTimePeriod _simpleTimePeriod = new SimpleTimePeriod(_date, _date_1);
        Task _task = new Task(_name, _simpleTimePeriod);
        s1.add(_task);
      }
    }
    TaskSeriesCollection collection = new TaskSeriesCollection();
    collection.add(s1);
    return ((IntervalCategoryDataset) collection);
  }
  
  /**
   * Creates a Gantt diagram according to the Gantt model in the parameter.
   */
  private JFreeChart createChart(final IntervalCategoryDataset dataset) {
    JFreeChart chart = ChartFactory.createGanttChart(
      "Execution results", 
      "State", 
      "Time", dataset, 
      true, 
      true, 
      false);
    return chart;
  }
  
  /**
   * Exports the Gantt diagram to the PDF file specified in the parameter.
   */
  public void saveChartToPDF(final JFreeChart chart, final String fileName, final int width, final int height) throws Exception {
    if ((chart != null)) {
      BufferedOutputStream out = null;
      try {
        FileOutputStream _fileOutputStream = new FileOutputStream(fileName);
        BufferedOutputStream _bufferedOutputStream = new BufferedOutputStream(_fileOutputStream);
        out = _bufferedOutputStream;
        Rectangle pagesize = new Rectangle(width, height);
        Document document = new Document(pagesize, 50, 50, 50, 50);
        try {
          PdfWriter writer = PdfWriter.getInstance(document, out);
          document.addAuthor("JFreeChart");
          document.open();
          PdfContentByte cb = writer.getDirectContent();
          PdfTemplate tp = cb.createTemplate(width, height);
          DefaultFontMapper _defaultFontMapper = new DefaultFontMapper();
          Graphics2D g2 = tp.createGraphics(width, height, _defaultFontMapper);
          Rectangle2D r2D = new Rectangle2D.Double(0, 0, width, height);
          chart.draw(g2, r2D, null);
          g2.dispose();
          cb.addTemplate(tp, 0, 0);
        } finally {
          document.close();
        }
      } finally {
        if ((out != null)) {
          out.close();
        }
      }
    }
  }
}
